@@ -9,7 +9,11 @@ import six
 from .utils import timestamp_from_datetime, timestamp_from_date, Money, DebitedBankAccount, Address, ShippingAddress, \
     Reason, ReportTransactionsFilters, ReportWalletsFilters, \
     PlatformCategorization, Billing, SecurityInfo, Birthplace, ApplepayPaymentData, GooglepayPaymentData, \
-    ScopeBlocked, BrowserInfo, Shipping, CurrentState, FallbackReason, InstantPayout, CountryAuthorizationData
+    ScopeBlocked, BrowserInfo, Shipping, CurrentState, FallbackReason, InstantPayout, CountryAuthorizationData, \
+    PayinsLinked, ConversionRate, CardInfo, LocalAccountDetails, InternationalAccountDetails, \
+    VirtualAccountCapabilities, PaymentRef, PendingUserAction, LegalRepresentative, IndividualRecipient, \
+    BusinessRecipient, RecipientPropertySchema, IndividualRecipientPropertySchema, BusinessRecipientPropertySchema, \
+    CompanyNumberValidation, ReportFilter, PayInIntentExternalData, PayInIntentBuyer, SupportedBank
 
 
 class FieldDescriptor(object):
@@ -159,7 +163,7 @@ class FloatField(Field):
 
 class DictField(Field):
     def api_value(self, value):
-        return json.dumps(value)
+        return value
 
     def python_value(self, value):
         if value is not None and isinstance(value, str):
@@ -205,9 +209,34 @@ class MoneyField(Field):
         value = super(MoneyField, self).api_value(value)
 
         if isinstance(value, Money):
+            if value.amount is not None:
+                value = {
+                    'Currency': value.currency,
+                    'Amount': int(value.amount)
+                }
+            else:
+                value = {
+                    'Currency': value.currency,
+                    'Amount': None
+                }
+
+        return value
+
+
+class PaymentRefField(Field):
+    def python_value(self, value):
+        if value is not None:
+            return PaymentRef(reason_type=value['ReasonType'], reference_id=value['ReferenceId'])
+
+        return value
+
+    def api_value(self, value):
+        value = super(PaymentRefField, self).api_value(value)
+
+        if isinstance(value, PaymentRef):
             value = {
-                'Currency': value.currency,
-                'Amount': int(value.amount)
+                'ReasonType': value.reason_type,
+                'ReferenceId': value.reference_id
             }
 
         return value
@@ -337,6 +366,8 @@ class ReportTransactionsFiltersField(Field):
         if value is not None:
             local_min_debited_funds_amount = ''
             local_max_debited_funds_amount = ''
+            local_min_fees_amount = ''
+            local_max_fees_amount = ''
 
             if 'MinDebitedFundsAmount' in value and value['MinDebitedFundsAmount']:
                 local_min_debited_funds_amount = int(value['MinDebitedFundsAmount'])
@@ -344,6 +375,12 @@ class ReportTransactionsFiltersField(Field):
             if 'MaxDebitedFundsAmount' in value and value['MaxDebitedFundsAmount']:
                 local_max_debited_funds_amount = int(value['MaxDebitedFundsAmount'])
 
+            if 'MinFeesAmount' in value and value['MinFeesAmount']:
+                local_min_fees_amount = int(value['MinFeesAmount'])
+
+            if 'MaxFeesAmount' in value and value['MaxFeesAmount']:
+                local_max_fees_amount = int(value['MaxFeesAmount'])
+
             author_id = None
             wallet_id = None
             if 'AuthorId' in value:
@@ -360,6 +397,11 @@ class ReportTransactionsFiltersField(Field):
                                              min_debited_funds_currency=value['MinDebitedFundsCurrency'],
                                              max_debited_funds_amount=local_max_debited_funds_amount,
                                              max_debited_funds_currency=value['MaxDebitedFundsCurrency'],
+                                             result_code=value['ResultCode'],
+                                             min_fees_amount=local_min_fees_amount,
+                                             min_fees_currency=value['MinFeesCurrency'],
+                                             max_fees_amount=local_max_fees_amount,
+                                             max_fees_currency=value['MaxFeesCurrency'],
                                              author_id=author_id, wallet_id=wallet_id
                                              )
         return value
@@ -389,6 +431,11 @@ class ReportTransactionsFiltersField(Field):
                 'MinDebitedFundsCurrency': value.min_debited_funds_currency,
                 'MaxDebitedFundsAmount': value.max_debited_funds_amount,
                 'MaxDebitedFundsCurrency': value.max_debited_funds_currency,
+                'ResultCode': value.result_code,
+                'MinFeesAmount': value.min_fees_amount,
+                'MinFeesCurrency': value.min_fees_currency,
+                'MaxFeesAmount': value.max_fees_amount,
+                'MaxFeesCurrency': value.max_fees_currency,
                 'AuthorId': value.author_id,
                 'WalletId': value.wallet_id,
             }
@@ -852,3 +899,422 @@ class CountryAuthorizationDataField(Field):
             }
 
         return value
+
+
+class PayinsLinkedField(Field):
+    def python_value(self, value):
+        if value is not None:
+            return PayinsLinked(payin_capture_id=value['PayinCaptureId'],
+                                payin_complement_id=value['PayinComplementId'])
+        return value
+
+    def api_value(self, value):
+        value = super(PayinsLinkedField, self).api_value(value)
+
+        if isinstance(value, PayinsLinked):
+            value = {
+                'PayinCaptureId': value.payin_capture_id,
+                'PayinComplementId': value.payin_complement_id
+            }
+
+        return value
+
+
+class ConversionRateField(Field):
+    def python_value(self, value):
+        if value is not None:
+            return ConversionRate(client_rate=value['ClientRate'], market_rate=value['MarketRate'])
+        return value
+
+    def api_value(self, value):
+        value = super(ConversionRateField, self).api_value(value)
+
+        if isinstance(value, ConversionRate):
+            value = {
+                'ClientRate': value.client_rate,
+                'MarketRate': value.market_rate
+            }
+
+        return value
+
+
+class CardInfoField(Field):
+    def python_value(self, value):
+        if value is not None:
+            return CardInfo(
+                bin=value['BIN'],
+                issuing_bank=value['IssuingBank'],
+                issuer_country_code=value['IssuerCountryCode'],
+                type=value['Type'],
+                brand=value['Brand'],
+                sub_type=value['SubType'])
+        return value
+
+    def api_value(self, value):
+        value = super(CardInfoField, self).api_value(value)
+
+        if isinstance(value, CardInfo):
+            value = {
+                'BIN': value.bin,
+                'IssuingBank': value.issuing_bank,
+                'IssuerCountryCode': value.issuer_country_code,
+                'Type': value.type,
+                'Brand': value.brand,
+                'SubType': value.sub_type,
+            }
+
+        return value
+
+
+class LocalAccountDetailsField(Field):
+    def python_value(self, value):
+        if value is not None:
+            return LocalAccountDetails(address=value['Address'], account=value['Account'], bank_name=value['BankName'])
+        return value
+
+    def api_value(self, value):
+        value = super(LocalAccountDetailsField, self).api_value(value)
+
+        if isinstance(value, LocalAccountDetails):
+            value = {
+                'Address': value.address,
+                'Account': value.account,
+                'BankName': value.bank_name
+            }
+
+        return value
+
+
+class InternationalAccountDetailsField(Field):
+    def python_value(self, value):
+        if value is not None:
+            return InternationalAccountDetails(address=value['Address'], account=value['Account'],
+                                               bank_name=value['BankName'])
+        return value
+
+    def api_value(self, value):
+        value = super(InternationalAccountDetailsField, self).api_value(value)
+
+        if isinstance(value, InternationalAccountDetails):
+            value = {
+                'Address': value.address,
+                'Account': value.account,
+                'BankName': value.bank_name
+            }
+
+        return value
+
+
+class VirtualAccountCapabilitiesField(Field):
+    def python_value(self, value):
+        if value is not None:
+            return VirtualAccountCapabilities(local_pay_in_available=value['LocalPayinAvailable'],
+                                              international_pay_in_available=value['InternationalPayinAvailable'],
+                                              currencies=value['Currencies'])
+        return value
+
+    def api_value(self, value):
+        value = super(VirtualAccountCapabilitiesField, self).api_value(value)
+
+        if isinstance(value, VirtualAccountCapabilities):
+            value = {
+                'LocalPayinAvailable': value.local_pay_in_available,
+                'InternationalPayinAvailable': value.international_pay_in_available,
+                'Currencies': value.currencies
+            }
+
+        return value
+
+
+class PendingUserActionField(Field):
+    def python_value(self, value):
+        if value is not None:
+            return PendingUserAction(redirect_url=value['RedirectUrl'])
+
+        return value
+
+    def api_value(self, value):
+        value = super(PendingUserActionField, self).api_value(value)
+
+        if isinstance(value, PendingUserAction):
+            value = {
+                'RedirectUrl': value.redirect_url
+            }
+
+        return value
+
+
+class LegalRepresentativeField(Field):
+    def python_value(self, value):
+        if value is not None:
+            return LegalRepresentative(first_name=value['FirstName'], last_name=value['LastName'],
+                                       birthday=value['Birthday'],
+                                       nationality=value['Nationality'],
+                                       country_of_residence=value['CountryOfResidence'],
+                                       email=value['Email'], phone_number=value['PhoneNumber'],
+                                       phone_number_country=value['PhoneNumberCountry'])
+
+        return value
+
+    def api_value(self, value):
+        value = super(LegalRepresentativeField, self).api_value(value)
+
+        if isinstance(value, LegalRepresentative):
+            value = {
+                'FirstName': value.first_name,
+                'LastName': value.last_name,
+                'Birthday': value.birthday,
+                'Nationality': value.nationality,
+                'CountryOfResidence': value.country_of_residence,
+                'Email': value.email,
+                'PhoneNumber': value.phone_number,
+                'PhoneNumberCountry': value.phone_number_country
+            }
+
+        return value
+
+
+class IndividualRecipientField(Field):
+    def python_value(self, value):
+        if value is not None:
+            return IndividualRecipient(first_name=value.get('FirstName', None), last_name=value.get('LastName', None),
+                                       address=value.get('Address', None))
+
+        return value
+
+    def api_value(self, value):
+        value = super(IndividualRecipientField, self).api_value(value)
+
+        if isinstance(value, IndividualRecipient):
+            value = {
+                'FirstName': value.first_name,
+                'LastName': value.last_name,
+                'Address': value.address
+            }
+
+        return value
+
+
+class BusinessRecipientField(Field):
+    def python_value(self, value):
+        if value is not None:
+            return BusinessRecipient(business_name=value.get('BusinessName', None), address=value.get('Address', None))
+
+        return value
+
+    def api_value(self, value):
+        value = super(BusinessRecipientField, self).api_value(value)
+
+        if isinstance(value, BusinessRecipient):
+            value = {
+                'BusinessName': value.business_name,
+                'Address': value.address
+            }
+
+        return value
+
+
+class RecipientPropertySchemaField(Field):
+    def python_value(self, value):
+        if value is not None:
+            return RecipientPropertySchema(required=value.get('Required', None),
+                                           max_length=value.get('MaxLength', None),
+                                           min_length=value.get('MinLength', None),
+                                           pattern=value.get('Pattern', None),
+                                           allowed_values=value.get('AllowedValues', None),
+                                           label=value.get('Label', None),
+                                           end_user_display=value.get('EndUserDisplay', None))
+
+        return value
+
+    def api_value(self, value):
+        value = super(RecipientPropertySchemaField, self).api_value(value)
+
+        if isinstance(value, RecipientPropertySchema):
+            value = {
+                'Required': value.required,
+                'MaxLength': value.max_length,
+                'MinLength': value.min_length,
+                'Pattern': value.pattern,
+                'AllowedValues': value.allowed_values,
+                'Label': value.label,
+                'EndUserDisplay': value.end_user_display
+            }
+
+        return value
+
+
+class IndividualRecipientPropertySchemaField(Field):
+    def python_value(self, value):
+        if value is not None:
+            return IndividualRecipientPropertySchema(first_name=value.get('FirstName', None),
+                                                     last_name=value.get('LastName', None),
+                                                     address=value.get('Address', None))
+
+        return value
+
+    def api_value(self, value):
+        value = super(IndividualRecipientPropertySchemaField, self).api_value(value)
+
+        if isinstance(value, IndividualRecipientPropertySchema):
+            value = {
+                'FirstName': value.first_name,
+                'LastName': value.last_name,
+                'Address': value.address
+            }
+
+        return value
+
+
+class BusinessRecipientPropertySchemaField(Field):
+    def python_value(self, value):
+        if value is not None:
+            return BusinessRecipientPropertySchema(business_name=value.get('BusinessName', None),
+                                                   address=value.get('Address', None))
+
+        return value
+
+    def api_value(self, value):
+        value = super(BusinessRecipientPropertySchemaField, self).api_value(value)
+
+        if isinstance(value, BusinessRecipientPropertySchema):
+            value = {
+                'BusinessName': value.business_name,
+                'Address': value.address
+            }
+
+        return value
+
+
+class CompanyNumberValidationField(Field):
+    def python_value(self, value):
+        if value is not None:
+            return CompanyNumberValidation(company_number=value.get('CompanyNumber', None),
+                                           country_code=value.get('CountryCode', None),
+                                           is_valid=value.get('IsValid', None),
+                                           validation_rules=value.get('ValidationRules', None))
+
+        return value
+
+    def api_value(self, value):
+        value = super(CompanyNumberValidationField, self).api_value(value)
+
+        if isinstance(value, CompanyNumberValidation):
+            value = {
+                'CompanyNumber': value.company_number,
+                'CountryCode': value.country_code,
+                'IsValid': value.is_valid,
+                'ValidationRules': value.validation_rules
+            }
+
+        return value
+
+
+class ReportFilterField(Field):
+    def python_value(self, value):
+        if value is not None:
+            return ReportFilter(currency=value.get('Currency', None), user_id=value.get('UserId', None),
+                                wallet_id=value.get('WalletId', None), payment_method=value.get('PaymentMethod', None),
+                                status=value.get('Status', None), type=value.get('Type', None),
+                                intent_id=value.get('IntentId', None),
+                                external_provider_name=value.get('ExternalProviderName', None),
+                                scheduled=value.get('Scheduled', None))
+
+        return value
+
+    def api_value(self, value):
+        value = super(ReportFilterField, self).api_value(value)
+
+        if isinstance(value, ReportFilter):
+            result = {}
+            if value.currency is not None:
+                result['Currency'] = value.currency
+            if value.user_id is not None:
+                result['UserId'] = value.user_id
+            if value.wallet_id is not None:
+                result['WalletId'] = value.wallet_id
+            if value.payment_method is not None:
+                result['PaymentMethod'] = value.payment_method
+            if value.status is not None:
+                result['Status'] = value.status
+            if value.type is not None:
+                result['Type'] = value.type
+            if value.intent_id is not None:
+                result['IntentId'] = value.intent_id
+            if value.external_provider_name is not None:
+                result['ExternalProviderName'] = value.external_provider_name
+            if value.scheduled is not None:
+                result['Scheduled'] = value.scheduled
+            return result
+
+        return value
+
+
+class PayInIntentExternalDataField(Field):
+    def python_value(self, value):
+        if value is not None:
+            return PayInIntentExternalData(external_processing_date=value.get('ExternalProcessingDate', None),
+                                           external_provider_reference=value.get('ExternalProviderReference', None),
+                                           external_merchant_reference=value.get('ExternalMerchantReference', None),
+                                           external_provider_name=value.get('ExternalProviderName', None),
+                                           external_provider_payment_method=value.get('ExternalProviderPaymentMethod',
+                                                                                      None))
+
+        return value
+
+    def api_value(self, value):
+        value = super(PayInIntentExternalDataField, self).api_value(value)
+
+        if isinstance(value, PayInIntentExternalData):
+            result = {}
+            if value.external_processing_date is not None:
+                result['ExternalProcessingDate'] = value.external_processing_date
+            if value.external_provider_reference is not None:
+                result['ExternalProviderReference'] = value.external_provider_reference
+            if value.external_merchant_reference is not None:
+                result['ExternalMerchantReference'] = value.external_merchant_reference
+            if value.external_provider_name is not None:
+                result['ExternalProviderName'] = value.external_provider_name
+            if value.external_provider_payment_method is not None:
+                result['ExternalProviderPaymentMethod'] = value.external_provider_payment_method
+            return result
+
+        return value
+
+
+class PayInIntentBuyerField(Field):
+    def python_value(self, value):
+        if value is not None:
+            return PayInIntentBuyer(id=value.get('Id', None))
+
+        return value
+
+    def api_value(self, value):
+        value = super(PayInIntentBuyerField, self).api_value(value)
+
+        if isinstance(value, PayInIntentBuyer):
+            result = {}
+            if value.id is not None:
+                result['Id'] = value.id
+            return result
+
+        return value
+
+
+class SupportedBanksField(Field):
+    def python_value(self, value):
+        if value is not None:
+            return SupportedBank(countries=value.get('Countries', None))
+
+        return value
+
+    def api_value(self, value):
+        value = super(SupportedBanksField, self).api_value(value)
+
+        if isinstance(value, SupportedBank):
+            result = {}
+            if value.countries is not None:
+                result['Countries'] = value.countries
+            return result
+
+        return value
