@@ -1,4 +1,6 @@
 # -*- coding: utf-8 -*-
+import random
+import string
 import time
 import unittest
 from datetime import date
@@ -10,9 +12,10 @@ from exam.decorators import fixture
 from mangopay import APIRequest
 from mangopay import get_default_handler
 from mangopay.auth import AuthorizationTokenManager, StaticStorageStrategy
-from mangopay.constants import LEGAL_USER_TYPE_CHOICES
-from mangopay.resources import BankAccount, Document, ReportTransactions, UboDeclaration, Ubo
-from mangopay.utils import Address, ReportTransactionsFilters, Birthplace, BrowserInfo
+from mangopay.resources import BankAccount, Document, ReportTransactions, UboDeclaration, Ubo, Deposit, DirectPayIn, \
+    VirtualAccount, NaturalUserSca, LegalUserSca, PayInIntent
+from mangopay.utils import Address, ReportTransactionsFilters, Birthplace, BrowserInfo, LegalRepresentative, \
+    PayInIntentLineItem, PayInIntentBuyer, PayInIntentSeller, PayInIntentExternalData
 from tests import settings
 from tests.mocks import RegisteredMocks
 from tests.resources import (NaturalUser, LegalUser, Wallet,
@@ -123,7 +126,7 @@ class BaseTest(RegisteredMocks):
         self.mock_tokenization_request()
 
         response = requests.post(card_registration.card_registration_url, data={
-            'cardNumber': '4970100000000154',
+            'cardNumber': '4970107111111119',
             'cardCvx': '123',
             'cardExpirationDate': '0124',
             'accessKeyRef': card_registration.access_key,
@@ -149,7 +152,7 @@ class BaseTest(RegisteredMocks):
         self.mock_tokenization_request()
 
         response = requests.post(card_registration.card_registration_url, data={
-            'cardNumber': '4970101122334422',
+            'cardNumber': '4970107111111119',
             'cardCvx': '123',
             'cardExpirationDate': '0124',
             'accessKeyRef': card_registration.access_key,
@@ -175,7 +178,7 @@ class BaseTest(RegisteredMocks):
         self.mock_tokenization_request()
 
         response = requests.post(card_registration.card_registration_url, data={
-            'cardNumber': '4970101122334406',
+            'cardNumber': '4970107111111119',
             'cardCvx': '123',
             'cardExpirationDate': '0124',
             'accessKeyRef': card_registration.access_key,
@@ -257,6 +260,8 @@ class BaseTest(RegisteredMocks):
 
 class BaseTestLive(unittest.TestCase):
     _john = None
+    _john_sca_payer = None
+    _john_sca_owner = None
     _johns_account = None
     _johns_wallet = None
     _johns_kyc_document = None
@@ -269,6 +274,8 @@ class BaseTestLive(unittest.TestCase):
     _johns_card_3dsecure = None
 
     _user_legal = None
+    _user_legal_sca_payer = None
+    _user_legal_sca_owner = None
     _ubo_declaration = None
     _ubo = None
 
@@ -277,31 +284,90 @@ class BaseTestLive(unittest.TestCase):
     def setUp(self):
         BaseTestLive.get_john()
         BaseTestLive.get_user_legal()
+        BaseTestLive.get_john_sca_payer()
+        BaseTestLive.get_john_sca_owner()
+        BaseTestLive.get_user_legal_sca_payer()
+        BaseTestLive.get_user_legal_sca_owner()
 
     @staticmethod
     def get_user_legal(recreate=False, terms=False):
         if BaseTestLive._user_legal is None or recreate:
-            legal = LegalUser()
-            legal.name = 'MatrixSampleOrg_PythonSDK'
-            legal.email = 'mail@test.com'
-            legal.legal_person_type = "BUSINESS"
-            legal.legal_representative_first_name = "Mango"
-            legal.legal_representative_last_name = 'Pay'
-            legal.legal_representative_email = 'mango@mangopay.com'
-            legal.person_type = 'LEGAL'
-            legal.headquarters_address = Address(address_line_1='AddressLine1', address_line_2='AddressLine2',
-                                                 city='City', region='Region',
-                                                 postal_code='11222', country='FR')
-            legal.legal_representative_birthday = 1300186358
-            legal.legal_representative_nationality = 'FR'
-            legal.legal_representative_country_of_residence = 'FR'
-            legal.company_number = 123456789
-            legal.tag = 'Python SDK Unit Test'
-            legal.terms_and_conditions_accepted = terms
-            legal.user_category = 'OWNER'
+            legal = BaseTestLive.get_user_legal_instance(terms)
             BaseTestLive._user_legal = LegalUser(**legal.save())
         return BaseTestLive._user_legal
 
+    @staticmethod
+    def get_user_legal_instance(terms=False):
+        legal = LegalUser()
+        legal.name = 'MatrixSampleOrg_PythonSDK'
+        legal.email = 'mail@test.com'
+        legal.legal_person_type = "BUSINESS"
+        legal.legal_representative_first_name = "Mango"
+        legal.legal_representative_last_name = 'Pay'
+        legal.legal_representative_email = 'mango@mangopay.com'
+        legal.person_type = 'LEGAL'
+        legal.headquarters_address = Address(address_line_1='AddressLine1', address_line_2='AddressLine2',
+                                             city='City', region='Region',
+                                             postal_code='11222', country='FR')
+        legal.legal_representative_birthday = 1300186358
+        legal.legal_representative_nationality = 'FR'
+        legal.legal_representative_country_of_residence = 'FR'
+        legal.company_number = 123456789
+        legal.tag = 'Python SDK Unit Test'
+        legal.terms_and_conditions_accepted = terms
+        legal.user_category = 'OWNER'
+        return legal
+
+    @staticmethod
+    def get_user_legal_sca_payer(recreate=False, terms=True):
+        if BaseTestLive._user_legal_sca_payer is None or recreate:
+            user = BaseTestLive.get_user_legal_sca_payer_instance(terms)
+            BaseTestLive._user_legal_sca_payer = LegalUserSca(**user.save())
+        return BaseTestLive._user_legal_sca_payer
+
+    @staticmethod
+    def get_user_legal_sca_payer_instance(terms=True):
+        user = LegalUserSca()
+        user.name = 'Alex Smith'
+        user.email = 'alex.smith.services@example.com'
+        user.legal_person_type = "SOLETRADER"
+        user.terms_and_conditions_accepted = terms
+        user.legal_representative = LegalRepresentative(first_name='Alex', last_name='Smith',
+                                                        email='alex.smith.services@example.com',
+                                                        phone_number='0611111111',
+                                                        phone_number_country='FR')
+        user.user_category = 'PAYER'
+        user.legal_representative_address = Address(address_line_1='AddressLine1', address_line_2='AddressLine2',
+                                                    city='City', region='Region',
+                                                    postal_code='11222', country='FR')
+        return user
+
+    @staticmethod
+    def get_user_legal_sca_owner(recreate=False, terms=True):
+        if BaseTestLive._user_legal_sca_owner is None or recreate:
+            user = LegalUserSca()
+            user.name = 'Alex Smith'
+            user.email = 'alex.smith.services@example.com'
+            user.legal_person_type = "SOLETRADER"
+            user.terms_and_conditions_accepted = terms
+            user.legal_representative = LegalRepresentative(first_name='Alex', last_name='Smith',
+                                                             email='alex.smith.services@example.com',
+                                                             phone_number='0611111111',
+                                                             phone_number_country='FR',
+                                                             birthday=652117514,
+                                                             nationality='FR',
+                                                             country_of_residence='FR')
+            user.headquarters_address = Address(address_line_1='AddressLine1', address_line_2='AddressLine2',
+                                                 city='City', region='Region',
+                                                 postal_code='11222', country='FR')
+            user.legal_representative_address = Address(address_line_1='AddressLine1', address_line_2='AddressLine2',
+                                                         city='City', region='Region',
+                                                         postal_code='11222', country='FR')
+            user.company_number = '123456789'
+            user.user_category = 'OWNER'
+            BaseTestLive._user_legal_sca_owner = LegalUserSca(**user.save())
+        return BaseTestLive._user_legal_sca_owner
+
     @staticmethod
     def get_ubo_declaration(recreate=False):
         if BaseTestLive._ubo_declaration is None or recreate:
@@ -382,11 +448,59 @@ class BaseTestLive(unittest.TestCase):
     @staticmethod
     def get_john(recreate=False, terms=False):
         if BaseTestLive._john is None or recreate:
-            user = NaturalUser()
-            user.first_name = 'John'
-            user.last_name = 'Doe'
+            user = BaseTestLive.get_john_instance(terms)
+            BaseTestLive._john = NaturalUser(**user.save())
+        return BaseTestLive._john
+
+    @staticmethod
+    def get_john_instance(terms=False):
+        user = NaturalUser()
+        user.first_name = 'John'
+        user.last_name = 'Doe'
+        user.birthday = 188352000
+        user.email = "john.doe@sample.org"
+        user.address = Address(address_line_1='AddressLine1', address_line_2='AddressLine2',
+                               city='City', region='Region',
+                               postal_code='11222', country='FR')
+        user.nationality = 'FR'
+        user.country_of_residence = 'FR'
+        user.occupation = 'programmer'
+        user.income_range = '1'
+        user.person_type = 'NATURAL'
+        user.terms_and_conditions_accepted = terms
+        user.user_category = 'OWNER'
+        return user
+
+    @staticmethod
+    def get_john_sca_payer(recreate=False, terms=True):
+        if BaseTestLive._john_sca_payer is None or recreate:
+            user = BaseTestLive.get_john_sca_payer_instance(terms)
+            BaseTestLive._john_sca_payer = NaturalUserSca(**user.save())
+        return BaseTestLive._john_sca_payer
+
+    @staticmethod
+    def get_john_sca_payer_instance(terms=True):
+        user = NaturalUserSca()
+        user.first_name = 'John SCA'
+        user.last_name = 'Doe SCA Review'
+        user.email = "john.doe.sca@sample.org"
+        user.terms_and_conditions_accepted = terms
+        user.user_category = 'PAYER'
+        user.phone_number = '+33611111111'
+        user.phone_number_country = 'FR'
+        user.address = Address(address_line_1='AddressLine1', address_line_2='AddressLine2',
+                               city='City', region='Region',
+                               postal_code='11222', country='FR')
+        return user
+
+    @staticmethod
+    def get_john_sca_owner(recreate=False, terms=True):
+        if BaseTestLive._john_sca_owner is None or recreate:
+            user = NaturalUserSca()
+            user.first_name = 'John SCA'
+            user.last_name = 'Doe SCA Review'
             user.birthday = 188352000
-            user.email = "john.doe@sample.org"
+            user.email = "john.doe.sca@sample.org"
             user.address = Address(address_line_1='AddressLine1', address_line_2='AddressLine2',
                                    city='City', region='Region',
                                    postal_code='11222', country='FR')
@@ -394,11 +508,12 @@ class BaseTestLive(unittest.TestCase):
             user.country_of_residence = 'FR'
             user.occupation = 'programmer'
             user.income_range = '1'
-            user.person_type = 'NATURAL'
             user.terms_and_conditions_accepted = terms
             user.user_category = 'OWNER'
-            BaseTestLive._john = NaturalUser(**user.save())
-        return BaseTestLive._john
+            user.phone_number = '+33611111111'
+            user.phone_number_country = 'FR'
+            BaseTestLive._john_sca_owner = NaturalUserSca(**user.save())
+        return BaseTestLive._john_sca_owner
 
     @staticmethod
     def get_johns_kyc_document(recreate=False):
@@ -412,23 +527,70 @@ class BaseTestLive(unittest.TestCase):
     @staticmethod
     def get_johns_wallet(recreate=False):
         if BaseTestLive._johns_wallet is None or recreate:
-            BaseTestLive._johns_wallet = BaseTestLive.create_new_wallet()
+            BaseTestLive._johns_wallet = BaseTestLive.create_new_wallet(BaseTestLive._john)
         return BaseTestLive._johns_wallet
 
     @staticmethod
-    def create_new_wallet():
+    def get_johns_wallet_with_money(user, recreate=False):
+        if BaseTestLive._johns_wallet is None or recreate:
+            BaseTestLive._johns_wallet = BaseTestLive.create_new_wallet_with_money(user)
+        return BaseTestLive._johns_wallet
+
+    @staticmethod
+    def create_new_wallet(user):
         wallet = Wallet()
-        wallet.owners = (BaseTestLive._john,)
+        wallet.owners = (user,)
         wallet.currency = 'EUR'
         wallet.description = 'WALLET IN EUR'
         return Wallet(**wallet.save())
 
+    @staticmethod
+    def create_new_wallet_with_money(user):
+        wallet = Wallet()
+        wallet.owners = (user,)
+        wallet.currency = 'EUR'
+        wallet.description = 'WALLET IN EUR'
+        wallet = Wallet(**wallet.save())
+
+        card_registration = CardRegistration()
+        card_registration.user = user
+        card_registration.currency = 'EUR'
+
+        saved_registration = card_registration.save()
+        data = {
+            'cardNumber': '4970107111111119',
+            'cardCvx': '123',
+            'cardExpirationDate': '1229',
+            'accessKeyRef': card_registration.access_key,
+            'data': card_registration.preregistration_data
+        }
+        headers = {
+            'content-type': 'application/x-www-form-urlencoded'
+        }
+        registration_data_response = requests.post(card_registration.card_registration_url, data=data, headers=headers)
+        saved_registration['registration_data'] = registration_data_response.text
+        updated_registration = CardRegistration(**saved_registration).save()
+        card_id = updated_registration['card_id']
+
+        direct_payin = DirectPayIn(author=user,
+                                   debited_funds=Money(amount=10000, currency='EUR'),
+                                   fees=Money(amount=0, currency='EUR'),
+                                   credited_wallet_id=wallet,
+                                   card_id=card_id,
+                                   secure_mode="DEFAULT",
+                                   ip_address="2001:0620:0000:0000:0211:24FF:FE80:C12C",
+                                   browser_info=BaseTest.get_browser_info(),
+                                   secure_mode_return_url="https://www.ulule.com/")
+
+        direct_payin.save()
+        return direct_payin.credited_wallet
+
     @staticmethod
     def get_johns_transfer(recreate=False):
         if BaseTestLive._johns_transfer is None or recreate:
             john = BaseTestLive.get_john()
             wallet1 = BaseTestLive.get_johns_wallet()
-            wallet2 = BaseTestLive.create_new_wallet()
+            wallet2 = BaseTestLive.create_new_wallet(john)
 
             transfer = Transfer()
             transfer.author = john
@@ -441,6 +603,26 @@ class BaseTestLive(unittest.TestCase):
             BaseTestLive._johns_transfer = Transfer(**transfer.save())
         return BaseTestLive._johns_transfer
 
+    @staticmethod
+    def get_johns_transfer_sca(sca_context, amount):
+        valid_john_sca_id = 'user_m_01JRG5WE99JYRCEBHZ8EKBRE8Y'
+        valid_matrix_sca_id = 'user_m_01JRG4ZWZ85RNZDKKTSFRMG6ZW'
+        john_sca = NaturalUserSca.get(valid_john_sca_id)
+        matrix_sca = LegalUserSca.get(valid_matrix_sca_id)
+
+        debited_wallet = BaseTestLive.create_new_wallet_with_money(john_sca)
+        credited_wallet = BaseTestLive.create_new_wallet(matrix_sca)
+
+        transfer = Transfer()
+        transfer.author = john_sca
+        transfer.credited_user = matrix_sca
+        transfer.debited_funds = Money(amount=amount, currency='EUR')
+        transfer.fees = Money(amount=0, currency='EUR')
+        transfer.debited_wallet = debited_wallet
+        transfer.credited_wallet = credited_wallet
+        transfer.sca_context = sca_context
+        return Transfer(**transfer.save())
+
     @staticmethod
     def get_johns_payout(recreate=False):
         if BaseTestLive._johns_payout is None or recreate:
@@ -488,16 +670,19 @@ class BaseTestLive(unittest.TestCase):
             card_registration = CardRegistration(**card_params)
             card_registration.save()
 
-            params = {
-                "data_XXX": card_registration.preregistration_data,
-                "accessKeyRef": card_registration.access_key,
-                "cardNumber": '4970101122334422',
-                "cardExpirationDate": '1224',
-                "cardCvx": '123'
+            data = {
+                'data': card_registration.preregistration_data,
+                'accessKeyRef': card_registration.access_key,
+                'cardNumber': '4970107111111119',
+                'cardExpirationDate': '1229',
+                'cardCvx': '123'
             }
-            response = APIRequest().custom_request('POST', card_registration.card_registration_url, None, None, False,
-                                                   False, **params)
-            card_registration.registration_data = response
+            headers = {
+                'content-type': 'application/x-www-form-urlencoded'
+            }
+            registration_data_response = requests.post(card_registration.card_registration_url, data=data,
+                                                       headers=headers)
+            card_registration.registration_data = registration_data_response.text
             card_registration.save()
             BaseTestLive._johns_card = card_registration.card
         return BaseTestLive._johns_card
@@ -512,16 +697,19 @@ class BaseTestLive(unittest.TestCase):
             card_registration = CardRegistration(**card_params)
             card_registration.save()
 
-            params = {
-                "data_XXX": card_registration.preregistration_data,
-                "accessKeyRef": card_registration.access_key,
-                "cardNumber": '4970105191923460',
-                "cardExpirationDate": '1224',
-                "cardCvx": '123'
+            data = {
+                'data': card_registration.preregistration_data,
+                'accessKeyRef': card_registration.access_key,
+                'cardNumber': '4970107111111119',
+                'cardExpirationDate': '1229',
+                'cardCvx': '123'
             }
-            response = APIRequest().custom_request('POST', card_registration.card_registration_url, None, None, False,
-                                                   False, **params)
-            card_registration.registration_data = response
+            headers = {
+                'content-type': 'application/x-www-form-urlencoded'
+            }
+            registration_data_response = requests.post(card_registration.card_registration_url, data=data,
+                                                       headers=headers)
+            card_registration.registration_data = registration_data_response.text
             card_registration.save()
             BaseTestLive._johns_card = card_registration.card
         return BaseTestLive._johns_card
@@ -530,6 +718,97 @@ class BaseTestLive(unittest.TestCase):
     def get_oauth_manager():
         return BaseTestLive._oauth_manager
 
+    @staticmethod
+    def create_new_deposit():
+        user = BaseTestLive.get_john()
+        debited_funds = Money(amount=1000, currency='EUR')
+        card_registration = BaseTestLive.create_new_card_registration_for_deposit()
+
+        params = {
+            "author_id": user.id,
+            "debited_funds": debited_funds,
+            "card_id": card_registration.card_id,
+            "secure_mode_return_url": "http://lorem",
+            "ip_address": "2001:0620:0000:0000:0211:24FF:FE80:C12C",
+            "browser_info": BaseTest.get_browser_info()
+        }
+
+        deposit = Deposit(**params)
+
+        return Deposit(**deposit.save())
+
+    @staticmethod
+    def create_new_card_registration_for_deposit():
+        card_params = {
+            "user": BaseTestLive.get_john(),
+            "currency": 'EUR'
+        }
+        card_registration = CardRegistration(**card_params)
+        card_registration.save()
+
+        data = {
+            'data': card_registration.preregistration_data,
+            'accessKeyRef': card_registration.access_key,
+            'cardNumber': '4970107111111119',
+            'cardExpirationDate': '1229',
+            'cardCvx': '123'
+        }
+        headers = {
+            'content-type': 'application/x-www-form-urlencoded'
+        }
+        registration_data_response = requests.post(card_registration.card_registration_url, data=data,
+                                                   headers=headers)
+        card_registration.registration_data = registration_data_response.text
+
+        return CardRegistration(**card_registration.save())
+
+    @staticmethod
+    def create_new_virtual_account():
+        BaseTestLive.get_john()
+        wallet = BaseTestLive.get_johns_wallet()
+
+        virtual_account = VirtualAccount()
+        virtual_account.wallet_id = wallet.id
+        virtual_account.country = 'FR'
+        virtual_account.virtual_account_purpose = 'COLLECTION'
+
+        return VirtualAccount(**virtual_account.save())
+
+    @staticmethod
+    def create_new_pay_in_intent_authorization():
+        user = BaseTestLive.get_john()
+        wallet = BaseTestLive.get_johns_wallet()
+
+        external_data = PayInIntentExternalData()
+        external_data.external_processing_date = 1728133765
+        external_data.external_provider_reference = ''.join(random.choices(string.ascii_letters + string.digits, k=10))
+        external_data.external_merchant_reference = 'Order-xyz-35e8490e-2ec9-4c82-978e-c712a3f5ba16'
+        external_data.external_provider_name = 'Stripe'
+        external_data.external_provider_payment_method = 'PAYPAL'
+
+        seller = PayInIntentSeller()
+        seller.wallet_id = wallet.id
+        seller.author_id = wallet.owners_ids[0]
+        seller.transfer_date = 1728133765
+
+        buyer = PayInIntentBuyer()
+        buyer.id = user.id
+
+        line_item = PayInIntentLineItem()
+        line_item.seller = seller
+        line_item.sku = 'item-123456'
+        line_item.quantity = 1
+        line_item.unit_amount = 1000
+        line_items = [line_item]
+
+        intent_authorization = PayInIntent()
+        intent_authorization.amount = 1000
+        intent_authorization.currency = 'EUR'
+        intent_authorization.external_data = external_data
+        intent_authorization.line_items = line_items
+        intent_authorization.buyer = buyer
+        return PayInIntent(**intent_authorization.create_authorization())
+
     def test_handler(self):
         api_url = "test_api_url"
         sandbox_url = "test_sandbox_url"
