@@ -8,7 +8,7 @@ try:
 except ImportError:
     import urllib as urlrequest
 
-from mangopay.utils import Money, Billing, Address, SecurityInfo
+from mangopay.utils import Money, Billing, Address, SecurityInfo, CardInfo
 
 from datetime import date
 
@@ -676,9 +676,9 @@ class PreAuthorizationsTestLive(BaseTestLive):
 
         saved_registration = card_registration.save()
         data = {
-            'cardNumber': '4970105191923460',
+            'cardNumber': '4970107111111119',
             'cardCvx': '123',
-            'cardExpirationDate': '1224',
+            'cardExpirationDate': '1229',
             'accessKeyRef': card_registration.access_key,
             'data': card_registration.preregistration_data
         }
@@ -729,9 +729,9 @@ class PreAuthorizationsTestLive(BaseTestLive):
 
         saved_registration = card_registration.save()
         data = {
-            'cardNumber': '4970105191923460',
+            'cardNumber': '4970107111111119',
             'cardCvx': '123',
-            'cardExpirationDate': '1224',
+            'cardExpirationDate': '1229',
             'accessKeyRef': card_registration.access_key,
             'data': card_registration.preregistration_data
         }
@@ -740,6 +740,7 @@ class PreAuthorizationsTestLive(BaseTestLive):
         }
         registration_data_response = requests.post(card_registration.card_registration_url, data=data, headers=headers)
         saved_registration['registration_data'] = registration_data_response.text
+        saved_registration['card_holder_name'] = "John Silver"
         updated_registration = CardRegistration(**saved_registration).save()
 
         card = Card.get(updated_registration['card_id'])
@@ -782,6 +783,7 @@ class PreAuthorizationsTestLive(BaseTestLive):
         payin.culture = 'fr'
         BaseTestLive._johns_payin = PreAuthorizedPayIn(**payin.save())
 
+        time.sleep(2)
         transactions = pre_authorization.get_transactions()
 
         self.assertIsNotNone(saved_pre_authorization)
@@ -790,3 +792,59 @@ class PreAuthorizationsTestLive(BaseTestLive):
         self.assertEqual(security_info.avs_result, "NO_CHECK")
         self.assertEqual(payin.status, "SUCCEEDED")
         self.assertEqual(transactions[0].status, "SUCCEEDED")
+        self.assertIsNotNone(card.card_holder_name)
+
+    def test_PreAuthorizations_CheckCardInfo(self):
+        user = BaseTestLive.get_john()
+        card_registration = CardRegistration()
+        card_registration.user = user
+        card_registration.currency = "EUR"
+
+        saved_registration = card_registration.save()
+        data = {
+            'cardNumber': '4970107111111119',
+            'cardCvx': '123',
+            'cardExpirationDate': '1229',
+            'accessKeyRef': card_registration.access_key,
+            'data': card_registration.preregistration_data
+        }
+        headers = {
+            'content-type': 'application/x-www-form-urlencoded'
+        }
+        registration_data_response = requests.post(card_registration.card_registration_url, data=data, headers=headers)
+        saved_registration['registration_data'] = registration_data_response.text
+        updated_registration = CardRegistration(**saved_registration).save()
+
+        card = Card.get(updated_registration['card_id'])
+        pre_authorization = PreAuthorization()
+        pre_authorization.card = card
+        pre_authorization.author = user
+        pre_authorization.debited_funds = Money()
+        pre_authorization.debited_funds.currency = "EUR"
+        pre_authorization.debited_funds.amount = 100
+        pre_authorization.remaining_funds = Money()
+        pre_authorization.remaining_funds.currency = "EUR"
+        pre_authorization.remaining_funds.amount = 100
+        pre_authorization.secure_mode_return_url = "http://www.example.com/"
+        pre_authorization.ip_address = "2001:0620:0000:0000:0211:24FF:FE80:C12C"
+        pre_authorization.browser_info = BaseTest.get_browser_info()
+        pre_authorization.payment_category = 'TelephoneOrder'
+
+        billing = Billing()
+        billing.address = Address()
+        billing.address.address_line_1 = "Main Street"
+        billing.address.address_line_2 = "no. 5 ap. 6"
+        billing.address.country = "FR"
+        billing.address.city = "Lyon"
+        billing.address.postal_code = "65400"
+        billing.last_name = "Doe"
+        billing.first_name = "John"
+        pre_authorization.billing = billing
+
+        saved_pre_authorization = pre_authorization.save()
+
+        self.assertIsNotNone(saved_pre_authorization)
+        card_info = saved_pre_authorization['card_info']
+        self.assertIsNotNone(card_info)
+        self.assertIsInstance(card_info, CardInfo)
+        self.assertEqual('TelephoneOrder', saved_pre_authorization['payment_category'])
\ No newline at end of file
